@@ -1,7 +1,9 @@
-"""
-scikit-fmm is a Python extension module which implements the fast
+# -*- mode: doctest -*-
+"""scikit-fmm is a Python extension module which implements the fast
 marching method.
 
+https://github.com/scikit-fmm/scikit-fmm
+
 The fast marching method is used to model the evolution of boundaries
 and interfaces in a variety of application areas. More specifically,
 the fast marching method is a numerical technique for finding
@@ -16,83 +18,38 @@ point x on the curve. The speed function is specified, and the time at
 which the contour crosses a point x is obtained by solving the
 equation.
 
-scikit-fmm is a simple module which provides functions to calculate
-the signed distance and travel time to an interface described by the
-zero contour of the input array phi.
+skfmm.distance(phi, dx=1.0, self_test=False, order=2, narrow=0.0,
+  periodic=False)
 
->>> import skfmm
->>> import numpy as np
->>> phi = np.ones((3, 3))
->>> phi[1, 1] = -1
->>> skfmm.distance(phi)
-array([[ 1.20710678,  0.5       ,  1.20710678],
-       [ 0.5       , -0.35355339,  0.5       ],
-       [ 1.20710678,  0.5       ,  1.20710678]])
+  Return the signed distance from the zero contour of the array phi.
 
->>> skfmm.travel_time(phi, speed = 3.0 * np.ones_like(phi))
-array([[ 0.40236893,  0.16666667,  0.40236893],
-       [ 0.16666667,  0.11785113,  0.16666667],
-       [ 0.40236893,  0.16666667,  0.40236893]])
+skfmm.travel_time(phi, speed, dx=1.0, self_test=False, order=2,
+  narrow=0.0, periodic=False)
 
-The input array can be of 1, 2, 3 or higher dimensions and can be a
-masked array. A function is provided to compute extension velocities.
+  Return the travel from the zero contour of the array phi given the
+  scalar velocity field speed.
 
-Documentation:
-    Release Version:     http://packages.python.org/scikit-fmm
-    Development Version: http://scikit-fmm.readthedocs.org/en/latest/
+skfmm.extension_velocities(phi, speed, dx=1.0, self_test=False,
+  order=2, ext_mask=None, narrow=0.0, periodic=False)
 
-PyPI: http://pypi.python.org/pypi/scikit-fmm
-
-Source Code: https://github.com/scikit-fmm/scikit-fmm
-
-Requirements: Numpy and a C/C++ compiler (gcc, MinGW, MSVC)
-
-Bugs, questions, patches, feature requests, discussion & cetera:
-  Email list: http://groups.google.com/group/scikit-fmm
-  Send an email to scikit-fmm+subscribe@googlegroups.com to subscribe.
-
-Installing:
- $ python setup.py install
-
-Testing (doctest):
-  $ python -c "import skfmm; skfmm.test()"
-
-Building documentation (required sphinx and numpydoc):
-  $ make html
-
-Version History:
-
-0.0.1: February 13 2012
-  Initial release
-
-0.0.2: February 26th 2012
-  Including tests and docs in source distribution. Minor changes to
-  documentation.
-
-0.0.3: August 4th 2012
-  Extension velocities.
-  Fixes for 64 bit platforms.
-  Optional keyword argument for point update order.
-  Bug reports and patches from three contributors.
-
-0.0.4: October 15th 2012
-  Contributions from Daniel Wheeler:
-   * Bug fixes in extension velocity.
-   * Many additional tests and migration to doctest format.
-   * Additional optional input to extension_velocities() for FiPy compatibly.
-
-0.0.5: May 12th 2014
-   * Fix for building with MSVC (Jan Margeta).
-   * Corrected second-order point update.
+  Extend the velocities defined at the zero contour of phi, in the
+  normal direction, to the rest of the domain. Extend the velocities
+  such that grad f_ext dot grad d = 0 where where f_ext is the
+  extension velocity and d is the signed distance function.
 
 
-:Copyright: Copyright 2014 The scikit-fmm team.
-:License: BSD-style license. See LICENSE.txt in the scipy source directory.
+:Copyright: Copyright 2017 The scikit-fmm team.
+:License: BSD-style license. See LICENSE.txt in the source directory.
+
 """
-__version__ = "0.0.5"
+
+from __future__ import print_function
+
+__version__ = "2019.1.30"
 __docformat__ = 'restructuredtext'
 
-from pfmm import distance, travel_time, extension_velocities
+from .pfmm import distance, travel_time, extension_velocities
+from .heap import heap
 
 def testing():
     r"""
@@ -109,15 +66,60 @@ def testing():
 
     >>> import numpy as np
 
-    >>> print np.allclose(distance((-1., -1., -1., -1., 1., 1., 1., 1.), dx=.5),
-    ...                   (-1.75, -1.25, -.75, -0.25, 0.25, 0.75, 1.25, 1.75))
+    >>> print(np.allclose(distance((-1., -1., -1., -1., 1., 1., 1., 1.), dx=.5),
+    ...                   (-1.75, -1.25, -.75, -0.25, 0.25, 0.75, 1.25, 1.75)))
     True
 
     Small dimensions.
 
     >>> dx = 1e-10
-    >>> print np.allclose(distance((-1., -1., -1., -1., 1., 1., 1., 1.), dx=dx),
-    ...                   np.arange(8) * dx - 3.5 * dx)
+    >>> print(np.allclose(distance((-1., -1., -1., -1., 1., 1., 1., 1.), dx=dx),
+    ...                   np.arange(8) * dx - 3.5 * dx))
+    True
+
+    **Bug Fix**
+
+    Test case for a bug in the upwind finite difference scheme for
+    negative phi. When computing finite differences we want to
+    preferentially use information from the frozen neighbors that
+    are closest to the zero contour in each dimension. This means
+    that we must compare absolute distances when checking neighbors
+    in the negative phi direction.
+
+    The error can result in incorrect values of the updated signed
+    distance function for regions close to the minimum contour of
+    the level set function, i.e. in the middle of holes.
+
+    To test we use a square matrix for the initial phi field that is
+    equal to -1 on the main diagonal and on the three diagonals above
+    and below this. The matrix is set to 1 everywhere else. The bug
+    results in errors in positions (1,1), (2,2), (3,3), (6,6), (7,7)
+    and (8,8) along the main diagonal.
+
+    This error occurs for first- and second-order updates. For
+    simplicity, we choose to only test the first-order update.
+
+    >>> phi = np.ones((10, 10))
+    >>> i,j = np.indices(phi.shape)
+    >>> phi[i==j-3] = -1
+    >>> phi[i==j-2] = -1
+    >>> phi[i==j-1] = -1
+    >>> phi[i==j] = -1
+    >>> phi[i==j+1] = -1
+    >>> phi[i==j+2] = -1
+    >>> phi[i==j+3] = -1
+    >>> phi = distance(phi, order=1)
+    >>> print(np.allclose(phi[1, 1], -2.70464, atol=1e-4))
+    True
+    >>> print(np.allclose(phi[2, 2], -2.50873, atol=1e-4))
+    True
+    >>> print(np.allclose(phi[3, 3], -2.47487, atol=1e-4))
+    True
+    >>> print(np.allclose(phi[6, 6], -2.47487, atol=1e-4))
+    True
+    >>> print(np.allclose(phi[7, 7], -2.50873, atol=1e-4))
+    True
+    >>> print(np.allclose(phi[8, 8], -2.70464, atol=1e-4))
     True
 
     **Bug Fix**
@@ -135,8 +137,8 @@ def testing():
     >>> sqrt = crossProd**2 *(dsq - (vbr - vml)**2)
     >>> sqrt = np.sqrt(max(sqrt, 0))
     >>> vmr = (top + sqrt) / dsq
-    >>> print np.allclose(distance(((-1., 1., -1.), (1., 1., 1.)), dx=(dx, dy), order=1),
-    ...                   ((vbl, vml, vbl), (vbr, vmr, vbr)))
+    >>> print(np.allclose(distance(((-1., 1., -1.), (1., 1., 1.)), dx=(dx, dy), order=1),
+    ...                   ((vbl, vml, vbl), (vbr, vmr, vbr))))
     True
 
     **Test Extension Field Calculation**
@@ -147,9 +149,9 @@ def testing():
     ...                                    [[-1, .5], [2., -1.]],
     ...                                    ext_mask=phi < 0,
     ...                                    dx=1., order=1)
-    >>> print np.allclose(phi, ((-tmp / 2, 0.5), (0.5, 0.5 + tmp)))
+    >>> print(np.allclose(phi, ((-tmp / 2, 0.5), (0.5, 0.5 + tmp))))
     True
-    >>> print np.allclose(ext, [[1.25, .5], [2., 1.25]])
+    >>> print(np.allclose(ext, [[1.25, .5], [2., 1.25]]))
     True
 
     >>> phi = np.array(((-1., 1., 1.), (1., 1., 1.), (1., 1., 1.)))
@@ -163,14 +165,14 @@ def testing():
     >>> v2 = 1.5
     >>> tmp1 = (v1 + v2) / 2 + np.sqrt(2. - (v1 - v2)**2) / 2
     >>> tmp2 = tmp1 + 1 / np.sqrt(2)
-    >>> print np.allclose(phi, ((-tmp / 2, 0.5, 1.5),
+    >>> print(np.allclose(phi, ((-tmp / 2, 0.5, 1.5),
     ...                         (0.5, 0.5 + tmp, tmp1),
-    ...                         (1.5, tmp1, tmp2)))
+    ...                         (1.5, tmp1, tmp2))))
     True
-    >>> print np.allclose(ext, ((1.25, 2., 2.),
+    >>> print(np.allclose(ext, ((1.25, 2., 2.),
     ...                         (.5, 1.25, 1.5456),
     ...                         (.5, 0.9544, 1.25)),
-    ...                   rtol = 1e-4)
+    ...                   rtol = 1e-4))
     True
 
     **Bug Fix**
@@ -181,7 +183,7 @@ def testing():
     vectors. In fact the two closest cells could have opposite
     normals.
 
-    >>> print np.allclose(distance((-1., 1., -1.)), (-0.5, 0.5, -0.5))
+    >>> print(np.allclose(distance((-1., 1., -1.)), (-0.5, 0.5, -0.5)))
     True
 
     Testing second order. This example failed with Scikit-fmm_.
@@ -194,9 +196,9 @@ def testing():
     ...           (-0.5, -0.35355339, 0.5, 1.45118446),
     ...           (0.5, 0.5, 0.97140452, 1.76215286),
     ...           (1.49923009, 1.45118446, 1.76215286, 2.33721352))
-    >>> print np.allclose(distance(phi),
+    >>> print(np.allclose(distance(phi),
     ...                   answer,
-    ...                   rtol=1e-9)
+    ...                   rtol=1e-9))
     True
 
     **A test for a bug in both LSMLIB and Scikit-fmm**
@@ -233,22 +235,22 @@ def testing():
     slightly different order so gets a seemingly better answer, but
     this is just chance.
 
-    >>> print np.allclose(phi, answer, rtol=1e-9)
+    >>> print(np.allclose(phi, answer, rtol=1e-9))
     True
 
     The following tests for the same issue but is a better test case
-    guarentted to fail.
+    guaranteed to fail.
 
     >>> phi = np.array([[-1,  1,  1,  1,  1, -1],
     ...                 [-1, -1, -1, -1, -1, -1],
     ...                 [-1, -1, -1, -1, -1, -1]])
 
     >>> phi = distance(phi)
-    >>> print phi[2, 2] == phi[2, 3]
+    >>> print(phi[2, 2] == phi[2, 3])
     True
 
     >>> phi = distance(phi)
-    >>> print phi[2, 2] == phi[2, 3]
+    >>> print(phi[2, 2] == phi[2, 3])
     True
 
     **Circle Example**
@@ -310,7 +312,7 @@ def testing():
     ...     MASK,  MASK, MASK, MASK, MASK, MASK, MASK, MASK, MASK, MASK, MASK), 'd')
 
     >>> phi[trialValues == MASK] = MASK
-    >>> print np.allclose(phi, trialValues)
+    >>> print(np.allclose(phi, trialValues))
     True
 
     **Square Example**
@@ -360,7 +362,7 @@ def testing():
     ...     -3. * dx / 2., -dx / 2., v4              , -dx / 2., -3. * dx / 2.,
     ...     v1           , val     , -dy / 2.        , val     , v1           ,
     ...     v3           , v2      , -3. * dy / 2.   , v2      , v3           ))
-    >>> print np.allclose(arr, phi)
+    >>> print(np.allclose(arr, phi))
     True
 
     **Assertion Errors**
@@ -466,13 +468,13 @@ def testing():
     >>> d, f_ext = extension_velocities(phi, speed, dx, self_test=True)
     >>> np.testing.assert_allclose(f_ext, 1.0)
 
-    Constant value marchout test
+    Constant value march-out test
 
     >>> speed[abs(Y)<0.3] = 10.0
     >>> d, f_ext = extension_velocities(phi, speed, dx, self_test=True)
     >>> np.testing.assert_allclose(f_ext, 10.0)
 
-    Test distance from extenstion
+    Test distance from extension
 
     >>> speed = np.ones_like(phi)
     >>> d, f_ext = extension_velocities(phi, speed, dx, self_test=True)
@@ -630,7 +632,7 @@ def testing():
 
     >>> assert diff > 635 and diff < 645
 
-    Test eikonal solution
+    Test Eikonal solution
 
     >>> N     = 50
     >>> X, Y  = np.meshgrid(np.linspace(-1, 1, N), np.linspace(-1, 1, N))
@@ -737,12 +739,144 @@ def testing():
       ...
     ValueError: phi must be a 1 to 12-D array of doubles
 
+    >>> from skfmm import heap
+    >>> h = heap(10,True)
+    >>> h.push(0,0.2)
+    0
+    >>> h.push(1,0.3)
+    1
+    >>> h.push(2,0.1)
+    2
+    >>> h.update(1, 0.01)
+    >>> h.pop()
+    (1, 0.01)
+    >>> h.pop()
+    (2, 0.1)
+    >>> h.pop()
+    (0, 0.2)
+    >>> h.empty()
+    True
+    >>> h.pop()
+    Traceback (most recent call last):
+      ...
+    RuntimeError: heap pop error: empty heap
+    <BLANKLINE>
+
+    Test narrow optional argument.
+
+    >>> phi = np.array([-1,-1,-1,1,1,1])
+    >>> d = distance(phi, narrow=1.0)
+    >>> d.data[2:-2]
+    array([-0.5,  0.5])
+    >>> assert (d.mask == [ True,  True, False, False,  True,  True]).all()
+    >>> N     = 50
+    >>> X, Y  = np.meshgrid(np.linspace(-1, 1, N), np.linspace(-1, 1, N))
+    >>> r     = 0.5
+    >>> dx    = 2.0 / (N - 1)
+    >>> phi   = (X) ** 2 + (Y) ** 2 - r ** 2
+    >>> exact = np.sqrt(X ** 2 + Y ** 2) - r
+
+    >>> d = distance(phi, dx)
+    >>> bandwidth=5*dx
+    >>> d2 = distance(phi, dx, narrow=bandwidth)
+    >>> np.testing.assert_allclose(d, exact, atol=dx)
+    >>> # make sure we get a normal array if there are no points outside the narrow band
+    >>> np.testing.assert_allclose(distance(phi, dx, narrow=1000*dx), exact, atol=dx)
+
+    >>> assert  (d2<bandwidth).all()
+    >>> assert type(d2) is np.ma.masked_array
+
+    >>> speed = np.ones_like(phi)
+    >>> speed[X>0] = 1.8
+    >>> t = travel_time(phi, speed, dx)
+    >>> t2 = travel_time(phi, speed, dx, narrow=bandwidth)
+    >>> assert type(t2) is np.ma.masked_array
+    >>> assert (t2<bandwidth).all()
+
+    >>> speed = np.ones_like(phi)
+    >>> speed = X + Y
+    >>> ed, ev = extension_velocities(phi, speed, dx)
+    >>> ed2, ev2 = extension_velocities(phi, speed, dx, narrow=bandwidth)
+    >>> assert type(ed2) is np.ma.masked_array
+    >>> assert type(ev2) is np.ma.masked_array
+    >>> assert (ed2<bandwidth).all()
+    >>> assert ed2.shape == ed.shape == ev.shape == ev2.shape
+    >>> distance(phi, dx, narrow=-1)
+    Traceback (most recent call last):
+      ...
+    ValueError: parameter "narrow" must be greater than or equal to zero.
+    >>> # make sure the narrow options works with an existing mask.
+    >>> mask = abs(X)<0.25
+    >>> d3 = distance(np.ma.masked_array(phi, mask), dx, narrow=bandwidth)
+    >>> assert (d3.mask[mask]==True).all()
+    >>> assert d3.mask.sum() > mask.sum()
+
+    Testing periodic argument
+
+    >>> X, Y = np.meshgrid(np.linspace(-2,2,200), np.linspace(-2,2,200))
+    >>> phi = -1*np.ones_like(X); phi[X**2+(Y-0.9)**2<0.5] = 1.0
+    >>> speed = np.ones_like(X); speed[(X-0.9)**2+Y**2<1.0] = 2.0
+    >>> np.allclose(distance(phi),distance(phi,periodic=False)) and np.allclose(distance(phi),distance(phi,periodic=(0,0)))
+    True
+    >>> np.allclose(travel_time(phi,speed),travel_time(phi,speed,periodic=False)) and np.allclose(travel_time(phi,speed),travel_time(phi,speed,periodic=(0,0)))
+    True
+    >>> phi = -1*np.ones_like(X); phi[X**2+Y**2<0.5] = 1.0
+    >>> speed = np.ones_like(X); speed[X**2+Y**2<1.0] = 2.0
+    >>> np.allclose(distance(phi),distance(phi,periodic=True)) and np.allclose(distance(phi),distance(phi,periodic=(1,1)))
+    True
+    >>> np.allclose(travel_time(phi,speed),travel_time(phi,speed,periodic=True)) and np.allclose(travel_time(phi,speed),travel_time(phi,speed,periodic=(1,1)))
+    True
+    >>> np.allclose(travel_time(phi,speed),travel_time(phi,speed,periodic=True)) and np.allclose(travel_time(phi,speed),travel_time(phi,speed,periodic=[1,1]))
+    True
+    >>> np.allclose(travel_time(phi,speed),travel_time(phi,speed,periodic=True)) and np.allclose(travel_time(phi,speed),travel_time(phi,speed,periodic=[True,True]))
+    True
+    >>> phi = -1*np.ones_like(X); phi[X**2+(Y-0.9)**2<0.5] = 1.0
+    >>> speed = np.ones_like(X); speed[(X-0.9)**2+Y**2<1.0] = 2.0
+    >>> np.allclose(distance(np.roll(phi,137,axis=0),periodic=True),np.roll(distance(phi,periodic=True),137,axis=0))
+    True
+    >>> np.allclose(travel_time(np.roll(phi,-77,axis=1),np.roll(speed,-77,axis=1),periodic=True),np.roll(travel_time(phi,speed,periodic=True),-77,axis=1))
+    True
+
+    >>> phi=[1,-1,1,1,1,1]
+    >>> speed=[4,1,2,2,2,2]
+    >>> np.allclose(extension_velocities(phi,speed)[1],(2.5,2.5,1.5,1.5,1.5,1.5))
+    True
+    >>> np.allclose(extension_velocities(phi,speed,periodic=True)[1],(2.5,2.5,1.5,1.5,1.5,2.5))
+    True
+
+
+    This is issue #18,
+
+    >>> phi = np.array([[ 1,  1, -1, -1, -1, -1,  1],
+    ...                 [ 3,  1, -1, -1, -1, -1,  1],
+    ...                 [ 3,  3,  1,  1,  1,  1,  3],
+    ...                 [ 3,  3,  3,  3,  3,  3,  3],
+    ...                 [ 3,  3,  3,  3,  3,  3,  3]])
+    >>> speed = np.array([[ 1.   ,  1.   ,  0.057,  0.128,  0.037,  0.039,  1.   ],
+    ...                   [ 1.   ,  1.   ,  0.199,  0.408,  0.997,  0.688,  1.   ],
+    ...                   [ 1.   ,  1.   ,  1.   ,  1.   ,  1.   ,  1.   ,  1.   ],
+    ...                   [ 1.   ,  1.   ,  1.   ,  1.   ,  1.   ,  1.   ,  1.   ],
+    ...                   [ 1.   ,  1.   ,  1.   ,  1.   ,  1.   ,  1.   ,  1.   ]])
+    >>> assert not travel_time(phi,speed)[0][3] == 0
+    Traceback (most recent call last):
+      File "/usr/lib/python2.7/doctest.py", line 1315, in __run
+        compileflags, 1) in test.globs
+      File "<doctest skfmm.testing[347]>", line 1, in <module>
+        assert not travel_time(phi,speed)[0][3] == 0
+      File "skfmm/pfmm.py", line 173, in travel_time
+        int(self_test), TRAVEL_TIME, order, narrow, periodic)
+    RuntimeError: Negative discriminant in time marcher quadratic.
     """
 
-def  test(verbose=None):
+def test(verbose=None):
     r"""
     Run all the doctests available.
     """
     import doctest
     import skfmm
-    doctest.testmod(skfmm, verbose=verbose)
+    fail0, test0 = doctest.testmod(skfmm,      verbose=verbose)
+    fail1, test1 = doctest.testfile("heap.py", verbose=verbose)
+
+    print ("Summary: {} tests run {} failures".format(test0+test1,
+                                                      fail0+fail1))
+    return fail0+fail1
