@@ -6,6 +6,7 @@
 #include "base_marcher.h"
 #include "heap.h"
 #include "math.h"
+#include <vector>
 
 extern "C" {
 
@@ -13,8 +14,10 @@ extern "C" {
 baseMarcher::baseMarcher(
   double *phi,      double *dx,   long *flag,
   double *distance, int     ndim, int *shape,
-  bool self_test,   int order)
+  bool self_test,   int order,    double narrow,
+  int periodic)
 {
+  narrow_     =   narrow;
   order_      =   order;
   error_      =   1;
   phi_        =   phi;
@@ -26,6 +29,7 @@ baseMarcher::baseMarcher(
   self_test_  =   self_test;
   heapptr_    =   0;
   heap_       =   0;
+  periodic_   =   periodic;
 
   for (int i=0; i<dim_; i++)
   {
@@ -113,82 +117,110 @@ void baseMarcher::solve()
     return;
   }
   int i=0;
+  std::vector<int> toFreeze;
+
   while (! heap_->empty())
   {
+    toFreeze.clear();
     i++;
     double  value   = 0;
     int     addr    = 0;
     heap_->pop(&addr, &value);
+
+    if ((narrow_ != 0) && (fabs(value) > narrow_))
+    {
+      break;
+    }
     flag_[addr]=Frozen;
     finalizePoint(addr, value);
+    toFreeze.push_back(addr);
 
-    for (int dim=0; dim<dim_; dim++)
+    bool done=false;
+    while (!done)
     {
-      for (int j=-1; j<2; j+=2) // each direction
+      if (! heap_->empty() && value == heap_->peek())
       {
-        int naddr = _getN(addr,dim,j,Frozen);
-        if (naddr!=-1 && flag_[naddr]!=Frozen)
-        {
-          if (flag_[naddr]==Narrow)
-          {
+        double  l_value   = 0;
+        int     l_addr    = 0;
+        heap_->pop(&l_addr, &l_value);
+        flag_[l_addr]=Frozen;
+        finalizePoint(l_addr, l_value);
+        toFreeze.push_back(l_addr);
+      }
+      else
+        done = true;
+    }
 
-            double d;
-            if (order_ == 2)
-              d =  updatePointOrderTwo(naddr);
-            else
-              d =  updatePointOrderOne(naddr);
-            if (d)
-            {
-              heap_->set(heapptr_[naddr],fabs(d));
-              distance_[naddr]=d;
-            }
-          }
-          else if (flag_[naddr]==Far)
-          {
-            double d;
-            if (order_ == 2)
-              d =  updatePointOrderTwo(naddr);
-            else
-              d =  updatePointOrderOne(naddr);
-            if (d)
-            {
-              distance_[naddr]=d;
-              flag_[naddr]=Narrow;
-              heapptr_[naddr] = heap_->push(naddr,fabs(d));
-            }
-          }
-        }
-        //==========================================================
-        // update the far point in the second order stencil
-        // "jump" over a Frozen point if needed
-        if (order_ == 2)
+    for (unsigned int k=0; k<toFreeze.size(); k++)
+    {
+      int addr = toFreeze[k];
+
+      for (int dim=0; dim<dim_; dim++)
+      {
+        for (int j=-1; j<2; j+=2) // each direction
         {
-          int local_naddr = _getN(addr,dim,j,Mask);
-          if (local_naddr!=-1 && flag_[local_naddr]==Frozen)
+          int naddr = _getN(addr,dim,j,Frozen);
+          if (naddr!=-1 && flag_[naddr]!=Frozen)
           {
-            int naddr2 = _getN(addr,dim,j*2,Frozen);
-            if (naddr2!=-1 && flag_[naddr2]==Narrow)
+            if (flag_[naddr]==Narrow)
             {
-              double d = updatePointOrderTwo(naddr2);
+              double d;
+              if (order_ == 2)
+                d =  updatePointOrderTwo(naddr);
+              else
+                d =  updatePointOrderOne(naddr);
               if (d)
               {
-                heap_->set(heapptr_[naddr2], fabs(d));
-                distance_[naddr2]=d;
+                heap_->set(heapptr_[naddr],fabs(d));
+                distance_[naddr]=d;
+              }
+            }
+            else if (flag_[naddr]==Far)
+            {
+              double d;
+              if (order_ == 2)
+                d =  updatePointOrderTwo(naddr);
+              else
+                d =  updatePointOrderOne(naddr);
+              if (d)
+              {
+                distance_[naddr]=d;
+                flag_[naddr]=Narrow;
+                heapptr_[naddr] = heap_->push(naddr,fabs(d));
               }
             }
           }
-        }
-        //==========================================================
-      } // for each direction
-    } // for each dimension
+          //==========================================================
+          // update the far point in the second order stencil
+          // "jump" over a Frozen point if needed
+          if (order_ == 2)
+          {
+            int local_naddr = _getN(addr,dim,j,Mask);
+            if (local_naddr!=-1 && flag_[local_naddr]==Frozen)
+            {
+              int naddr2 = _getN(addr,dim,j*2,Frozen);
+              if (naddr2!=-1 && flag_[naddr2]==Narrow)
+              {
+                double d = updatePointOrderTwo(naddr2);
+                if (d)
+                {
+                  heap_->set(heapptr_[naddr2], fabs(d));
+                  distance_[naddr2]=d;
+                }
+              }
+            }
+          }
+          //==========================================================
+        } // for each direction
+      } // for each dimension
+    }
   } // main loop of Fast Marching Method
 
   // add back mask here. The python wrapper will look for elements
   // equal to maxDouble and add the mask back
   for (int i=0; i<size_; i++)
   {
-    if (flag_[i] == Mask) distance_[i] = maxDouble;
-    if (flag_[i] == Far)  distance_[i] = maxDouble;
+    if (flag_[i] != Frozen) distance_[i] = maxDouble;
   }
   error_ = 0;
   return;
