@@ -8,6 +8,8 @@
 #include "travel_time_marcher.h"
 #include "extension_velocity_marcher.h"
 
+#include <stdexcept>
+
 #define DISTANCE              0
 #define TRAVEL_TIME           1
 #define EXTENSION_VELOCITY    2
@@ -23,14 +25,45 @@ static PyMethodDef fmm_methods[] =
     {NULL, NULL, 0, NULL}
 };
 
-PyMODINIT_FUNC initcfmm(void)
+#if PY_MAJOR_VERSION >= 3
+static struct PyModuleDef moduledef = {
+    PyModuleDef_HEAD_INIT,
+    "cfmm",                               /* m_name */
+    "c extension module for scikit-fmm",  /* m_doc */
+    -1,                                   /* m_size */
+    fmm_methods,                          /* m_methods */
+    NULL,                                 /* m_reload */
+    NULL,                                 /* m_traverse */
+    NULL,                                 /* m_clear */
+    NULL,                                 /* m_free */
+};
+#endif
+
+PyMODINIT_FUNC
+#if PY_MAJOR_VERSION >= 3
+PyInit_cfmm(void)
+#else
+initcfmm(void)
+#endif
 {
     PyObject* m;
+
+#if PY_MAJOR_VERSION >= 3
+    m = PyModule_Create(&moduledef);
+
+    if (m == NULL)
+        return NULL;
+
+    import_array();
+    return m;
+#else
     m = Py_InitModule3("cfmm", fmm_methods,
         "c extension module for scikit-fmm");
     if (m == NULL)
         return;
+
     import_array();
+#endif
 }
 
 static PyObject *distance_method(PyObject *self, PyObject *args)
@@ -39,15 +72,20 @@ static PyObject *distance_method(PyObject *self, PyObject *args)
   // -- phi, dx, flag, and speed
   // -- and the input error checking should be done
 
-  PyObject *pphi, *pdx, *pflag, *pspeed;
-  int       self_test, mode, order;
-  PyArrayObject *phi, *dx, *flag, *speed, *distance, *f_ext;
+  PyObject *pphi, *pdx, *pflag, *pspeed, *pext_mask;
+  int       self_test, mode, order, periodic;
+  PyArrayObject *phi, *dx, *flag, *speed, *distance, *f_ext, *ext_mask;
+  double narrow=0;
   distance = 0;
   f_ext    = 0;
   speed    = 0;
+  ext_mask = 0;
 
-  if (!PyArg_ParseTuple(args, "OOOOiii", &pphi, &pdx, &pflag,
-                        &pspeed, &self_test, &mode, &order))
+
+
+  if (!PyArg_ParseTuple(args, "OOOOOiiidi", &pphi, &pdx, &pflag,
+                        &pspeed, &pext_mask, &self_test, &mode,
+                        &order, &narrow, &periodic))
   {
     return NULL;
   }
@@ -182,13 +220,28 @@ static PyObject *distance_method(PyObject *self, PyObject *args)
     f_ext = (PyArrayObject *)PyArray_ZEROS(PyArray_NDIM(phi),
                                            shape2, PyArray_DOUBLE, 0);
     if (! f_ext) return NULL;
-  }
 
+    ext_mask = (PyArrayObject *)PyArray_FROMANY(pext_mask, PyArray_LONG, 1,
+                                                10, NPY_IN_ARRAY);
+    if (! ext_mask)
+      {
+        PyErr_SetString(PyExc_ValueError,
+                        "ext_mask must be a 1D, 12-D array of integers");
+        Py_XDECREF(phi);
+        Py_XDECREF(dx);
+        Py_XDECREF(flag);
+        Py_XDECREF(speed);
+        return NULL;
+      }
+
+  }
 
   // create a level set object to do the calculation
   double * local_phi        = (double *) PyArray_DATA(phi);
   double * local_dx         = (double *) PyArray_DATA(dx);
-  long   * local_flag       = (long *)    PyArray_DATA(flag);
+  long   * local_flag       = (long *)   PyArray_DATA(flag);
+  long    * local_ext_mask   = 0;
+  if (ext_mask) local_ext_mask = (long *) PyArray_DATA(ext_mask);
   double * local_speed      = 0;
   if (speed) local_speed    = (double *) PyArray_DATA(speed);
   double * local_distance   = (double *) PyArray_DATA(distance);
@@ -207,7 +260,9 @@ static PyObject *distance_method(PyObject *self, PyObject *args)
         PyArray_NDIM(phi),
         shape,
         self_test,
-        order);
+        order,
+        narrow,
+        periodic);
     }
     break;
     case TRAVEL_TIME:
@@ -221,7 +276,9 @@ static PyObject *distance_method(PyObject *self, PyObject *args)
         shape,
         self_test,
         order,
-        local_speed);
+        local_speed,
+        narrow,
+        periodic);
     }
     break;
     case EXTENSION_VELOCITY:
@@ -236,22 +293,36 @@ static PyObject *distance_method(PyObject *self, PyObject *args)
         shape,
         self_test,
         order,
+        local_ext_mask,
         local_speed,
-        local_fext);
+        local_fext,
+        narrow,
+        periodic);
     }
     break;
   default: error=1;
   }
 
-  marcher->march();
-  error = marcher->getError();
-  delete marcher;
-
+  try {
+      marcher->march();
+      error = marcher->getError();
+      delete marcher;
+    } catch (const std::exception& exn) {
+      // propagate error
+      PyErr_SetString(PyExc_RuntimeError, exn.what());
+      Py_XDECREF(phi);
+      Py_XDECREF(dx);
+      Py_XDECREF(flag);
+      Py_XDECREF(speed);
+      Py_XDECREF(ext_mask);
+      return NULL;
+    }
 
   Py_DECREF(phi);
   Py_DECREF(flag);
   Py_DECREF(dx);
   Py_XDECREF(speed);
+  Py_XDECREF(ext_mask);
 
   switch (error)
   {
