@@ -3,6 +3,7 @@
 #include "travel_time_marcher.h"
 #include "math.h"
 #include "heap.h"
+#include <stdexcept>
 
 void travelTimeMarcher::initalizeFrozen()
 {
@@ -35,7 +36,7 @@ double travelTimeMarcher::updatePointOrderTwo(int i)
       naddr = _getN(i,dim,j,Mask);
       if (naddr!=-1 && flag_[naddr]==Frozen)
       {
-        if (distance_[naddr]<value1)
+        if (fabs(distance_[naddr])<fabs(value1))
         {
           value1 = distance_[naddr];
           int naddr2 = _getN(i,dim,j*2,Mask);
@@ -76,15 +77,13 @@ double travelTimeMarcher::solveQuadratic(int i, const double &a,
   c -= 1/pow(speed_[i],2);
   double r0=0;
   double det = pow(b,2)-4*a*c;
-  if (det>0)
+  if (det>=0)
   {
     r0 = (-b+sqrt(det))/2.0/a;
   }
   else
   {
-    return 0;
+    throw std::runtime_error("Negative discriminant in time marcher quadratic.");
   }
   return r0;
 }
-
-
